#ifndef __POINT_H__
#define __POINT_H__

#include "mathlib_defs.h"
#include "genlib.h"

//////////////////////////////////////////////////
// Operations Supported
//
// Point p, q, r;
// Vector u, v, w;   // Vectors
// FLOAT f;          // Scalars
// BOOL b;           // Booleans
//
// f = p[0];       // LHS Index Operation
// p[0] = f;       // LHS Index Operation
// f = p(0);       // RHS Index Operation
// p(0) = f;       // ILLEGAL -- RHS (right hand side) only!!!
//
// p = q;
// p = v;
// p += v;
// p -= v;
// p = q + v;
// v = p - q;
// f = p.Dot(v);   // Dot Product for on Plane calculations

// Operations for Averaging to find Centroids and Doing Linear Combinations
// p += q;
// p *= f;         // Scalar Multiplication
// p /= f;         // Scalar Multiplication
// p = q + r;
// p = q * f;      // Scalar Multiplication
// p = f * q;      // Scalar Multiplication
// p = q / f;      // Scalar Division
// p = f / q;      // Scalar Inversion
//

// Logic
// b = (p == q); // Quasi Equal allows for floating point inaccuracy
// b = (p != q); // Quasi Not Equal allows for floating point inaccuracy

// Bounding Box Checking Operations
// b = (p < q);  // Elementwise
// b = (p <= q); // Elementwise
// b = (p > q);  // Elementwise
// b = (p >= q); // Elementwise

//////////////////////////////////////////////////////////////////////
// Point Class
//

class Vector;

class Point
{
public:
  MATHLIB_API Point();
  MATHLIB_API Point(FLOAT x, FLOAT y, FLOAT z, FLOAT w = 1.0f);
  MATHLIB_API virtual ~Point();

public:
  MATHLIB_API inline VOID Set(FLOAT x, FLOAT y, FLOAT z, FLOAT w = 1.0f);
  MATHLIB_API inline VOID Set(const POINT4D pt);
  MATHLIB_API inline VOID Get(POINT4D pt) const;
  MATHLIB_API inline const FLOAT *Get() const;

public:
  MATHLIB_API VOID Null();
  MATHLIB_API BOOL Homogenize();

public:
  MATHLIB_API inline FLOAT &operator[](const UINT_32 uiIndex);
  MATHLIB_API inline FLOAT operator()(const UINT_32 uiIndex) const;

  MATHLIB_API Point &operator=(const Point &pt);
  MATHLIB_API Point &operator=(const Vector &v);
  MATHLIB_API Point &operator+=(const Vector &v);
  MATHLIB_API Point &operator-=(const Vector &v);

  MATHLIB_API Point &operator+=(const Point &pt);
  MATHLIB_API Point &operator*=(const FLOAT f);
  MATHLIB_API Point &operator/=(const FLOAT f);

  MATHLIB_API BOOL operator==(const Point &pt) const;
  MATHLIB_API BOOL operator!=(const Point &pt) const;

  MATHLIB_API FLOAT Dot(const Vector &v) const;

  MATHLIB_API BOOL operator<(const Point &pt) const;
  MATHLIB_API BOOL operator<=(const Point &pt) const;
  MATHLIB_API BOOL operator>(const Point &pt) const;
  MATHLIB_API BOOL operator>=(const Point &pt) const;

protected:
  MATHLIB_API FLOAT Dot(const Point &pt) const; // For Equality Checking

public:
  MATHLIB_API VOID Display() const;

protected:
  POINT4D m_pt;
};

//////////////////////////////////////////////////////////////////////
// External Operators
//

MATHLIB_API inline Point operator+(const Point &pt, const Vector &v);
MATHLIB_API inline Vector operator-(const Point &pt, const Point &q);

MATHLIB_API inline Point operator+(const Point &pt, const Point &q);
MATHLIB_API inline Point operator*(const Point &pt, const FLOAT f);
MATHLIB_API inline Point operator*(const FLOAT f, const Point &pt);
MATHLIB_API inline Point operator/(const Point &pt, const FLOAT f);

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

MATHLIB_API inline VOID Point::Set(FLOAT x, FLOAT y, FLOAT z, FLOAT w/* = 1.0f*/)
{
  m_pt[X] = x;
  m_pt[Y] = y;
  m_pt[Z] = z;
  m_pt[W] = w;
}

MATHLIB_API inline VOID Point::Set(const POINT4D pt)
{
  ASSERT( pt != NULL );

  memcpy(m_pt, pt, sizeof(POINT4D));
}

MATHLIB_API inline VOID Point::Get(POINT4D pt) const
{
  ASSERT( pt != NULL );

  memcpy(pt, m_pt, sizeof(POINT4D));
}

MATHLIB_API inline const FLOAT *Point::Get() const
{
  return m_pt;
}

MATHLIB_API inline FLOAT &Point::operator[](const UINT_32 uiIndex)
{
  return m_pt[uiIndex];
}

MATHLIB_API inline FLOAT Point::operator()(const UINT_32 uiIndex) const
{
  return m_pt[uiIndex];
}

//////////////////////////////////////////////////////////////////////
// External Operator Implementations
//

#include "vector.h"

MATHLIB_API inline Point operator+(const Point &pt, const Vector &v)
{
  return Point(pt(0) + v(0), pt(1) + v(1), pt(2) + v(2), pt(3) + v(3));
}

MATHLIB_API inline Vector operator-(const Point &pt, const Point &q)
{
  return Vector(pt(0) - q(0), pt(1) - q(1), pt(2) - q(2), pt(3) - q(3));
}

MATHLIB_API inline Point operator+(const Point &pt, const Point &q)
{
  return Point(pt(0) + q(0), pt(1) + q(1), pt(2) + q(2), pt(3) + q(3));
}

MATHLIB_API inline Point operator*(const Point &pt, const FLOAT f)
{
  return Point(pt(0)*f, pt(1)*f, pt(2)*f, pt(3)*f);  
}

MATHLIB_API inline Point operator*(const FLOAT f, const Point &pt)
{
  return Point(pt(0)*f, pt(1)*f, pt(2)*f, pt(3)*f);  
}

MATHLIB_API inline Point operator/(const Point &pt, const FLOAT f)
{
  ASSERT( f != 0.0 );

  return Point(pt(0)/f, pt(1)/f, pt(2)/f, pt(3)/f);  
}

#endif // __POINT_H__

