#ifndef __SYMMETRYGROUP_H__
#define __SYMMETRYGROUP_H__

#include "mathlib.h"
#include "estypes.h"
#include "tile.h"

#include <GL/gl.h>
#include <GL/glu.h>

//////////////////////////////////////////////////////////////////////////////
// SymmetryGroup
//

class SymmetryGroup
{

//////////////////////////////////////////////////
// Initialization
//
public:
  SymmetryGroup();
  virtual ~SymmetryGroup();

  virtual VOID Init();

  virtual VOID InitFaceGrid() = 0;
  virtual VOID InitTileGrid();

  virtual VOID InitSphereFaceGrid() = 0;
  virtual VOID InitSphereTileGrid();
  virtual VOID InitOutline() = 0;

  virtual VOID InitFaceMatrices() = 0;
  virtual VOID InitTileMatrices() = 0;

//////////////////////////////////////////////////
// Rendering
//
public:
  virtual VOID RenderPolyhedra();
  virtual VOID RenderBall();
  virtual VOID RenderStereographicPoints();
  virtual VOID RenderSphereTriangulation();
  virtual VOID RenderSolid();

  VOID RenderSelectionSolid();

  virtual VOID RenderSelectionSphereTileOutlinePoints();
  virtual VOID RenderSelectionSphereTileOutlineEdges();

  virtual VOID RenderSelectionSphereTileInnerPoints();
  virtual VOID RenderSelectionSphereTileInnerEdges();

  virtual VOID RenderSelectionSphereTileHeightOutlinePoints();
  virtual VOID RenderSelectionSphereTileHeightInnerPoints();

  virtual VOID RenderSelectionSphereTileHolePoints();

//////////////////////////////////////////////////
// Output
//
public:
  virtual VOID OutputES(FILE *fpOutputFile) = 0;
  virtual VOID OutputESColor(FILE *fpOutputFile);

  VOID OutputSTL(FILE *fpOutputFile);
  VOID OutputVRML(FILE *fpOutputFile);

  VOID ClearSphereOutlinePoints();
  VOID ClearSphereInnerPoints();
  VOID ClearSphereHolePoints();
  VOID CreateTilePoint(Point pt, UINT_32 uiType, UINT_32 uiTilePoint, 
		       UINT_32 uiConnected, FLOAT fHeight, Matrix m);
  VOID SetTilePointPartners(UINT_32 uiID, UINT_32 uiIDPartner);
  VOID SetTileHeightPartnersFromOpenES(UINT_32 uiID, UINT_32 uiIDPartner);
  VOID TesselateTileOutlinePoints();
  VOID TesselateTileInnerPoints();

//////////////////////////////////////////////////
// Functions
//
public:
  virtual VOID AddSphereFaceGridTesselatedPoints(Point pt1, Point pt2);
  virtual VOID AddSphereTileGridTesselatedPoints(Point pt1, Point pt2);

  virtual VOID MoveOutlinePoint(UINT_32 uiSelectedPoint, Point ptLocation);
  virtual VOID InsertOutlinePoint(UINT_32 uiSelectedPoint, Point ptLocation);
  virtual VOID DeleteOutlinePoint(UINT_32 uiSelectedPoint);

  virtual TilePoint *AddInnerPoint(Point ptLocation);
  virtual TilePoint *AddInnerPointCopy(UINT_32 uiSelectedType, UINT_32 uiSelectedPoint);
  virtual VOID MoveInnerPoint(UINT_32 uiSelectedPoint, Point ptLocation);
  virtual VOID InsertInnerPoint(UINT_32 uiSelectedPoint, Point ptLocation);
  virtual VOID DeleteInnerPoint(UINT_32 uiSelectedPoint);
  virtual VOID SnapInnerPointToSelected(UINT_32 uiInnerPointIndex, UINT_32 uiSelectedType,
					UINT_32 uiSelectedIndex);
  virtual VOID ClearSnapPoint(UINT_32 uiInnerPointIndex);

  virtual VOID AddHolePoint(Point ptLocation);
  virtual VOID MoveHolePoint(UINT_32 uiSelectedPoint, Point ptLocation);
  virtual VOID DeleteHolePoint(UINT_32 uiSelectedPoint);

  virtual BOOL GetHeightSelected(UINT_32 uiSelectedType, UINT_32 uiSelectedPoint);
  virtual VOID SetHeightSelected(UINT_32 uiSelectedType, UINT_32 uiSelectedPoint, BOOL b);
  virtual VOID TogleHeightSelected(UINT_32 uiSelectedType, UINT_32 uiSelectedPoint);
  Vector GetPointNormalVector(UINT_32 uiSelected);
  VOID AddHeightToPoints(FLOAT fHeight);
  VOID DeselectAllHeightPoints();

  virtual VOID FindMatrices(Matrix &m1, Matrix &m2, TilePoint *tpt1) = 0;

  virtual VOID CalculateStereographicPoints();
  virtual VOID TriangulateTile();

  VOID PaintTile(UINT_32 uiSelectedTile, Vector vColor);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  VOID SetRadius(FLOAT fRadius);
  VOID SetThickness(FLOAT fThickness);

  inline VOID SetAllSolidTiles(BOOL bAll);
  inline VOID SetTileSolidModified(BOOL b);
  inline VOID SetTileBasReliefModified(BOOL b);

//////////////////////////////////////////////////
// Class Variables
//
protected:

//////////////////////////////////////////////////
// Member Variables
//
protected:

  GLuint GLuiSolid;
  GLuint GLuiBasRelief;
  BOOL m_bMakeSolidDisplayList;
  BOOL m_bMakeBasReliefDisplayList;

  BOOL m_bAllSolidTiles;
  BOOL m_bTriangulated;
  Tile *m_pTile;

  FLOAT m_fDihedralAngle;  

  // Tile points in counterclockwise order
  Point m_pt2DTilePoint1;
  Point m_pt2DTilePoint2;
  Point m_pt2DTilePoint3;
  Point m_pt2DTilePoint4;
  Point m_pt2DTilePoint5;
  Point m_pt2DTilePoint6;

  CListInt m_listTileTransformMatrix;  
  CListInt m_listFaceTransformMatrix;  

  CListInt m_listTileColor;
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline VOID SymmetryGroup::SetAllSolidTiles(BOOL bAll)
{
  m_bAllSolidTiles = bAll;
  SetTileSolidModified(TRUE);
}

inline VOID SymmetryGroup::SetTileSolidModified(BOOL b)
{
  m_pTile->SetSolidModified(b);
}

inline VOID SymmetryGroup::SetTileBasReliefModified(BOOL b)
{
  m_pTile->SetBasReliefModified(b);
}

#endif // __SYMMETRYGROUP_H__
