#ifndef __VECTOR_H__
#define __VECTOR_H__

#include "mathlib_defs.h"

#include "genlib.h"

//////////////////////////////////////////////////
// Operations Supported
//
// Vector u, v, w;   // Vectors
// Point p;          // Points
// FLOAT f;          // Scalars
// BOOL b;           // Booleans
//
// f = v[0];       // LHS Index Operation
// v[0] = f;       // LHS Index Operation
// f = v(0);       // RHS Index Operation
// v(0) = f;       // ILLEGAL -- RHS (right hand side) only!!!
//
// u = v;
// u = p;
// u += v;
// u -= v;
// u *= f;         // Scalar Multiplication
// u /= f;         // Scalar Division
// u = -v;
// u = v + w;
// u = v - w;
// u = v * w;      // Cross Product
// u = v.Cross(w); // Cross Product
// f = v.Dot(w);   // Dot Product
// f = v.Dot(p);   // Dot Product for on Plane Calculations
// u = v * f;      // Scalar Multiplication
// u = f * v;      // Scalar Multiplication
// u = v / f;      // Scalar Division
// u = f / v;      // Scalar Inversion
//
// Logic
// b = (u == v); // Quasi Equal allows for floating point inaccuracy
// b = (u != v); // Quasi Not Equal allows for floating point inaccuracy

//////////////////////////////////////////////////////////////////////
// Vector Class
//

class Point;

class Vector
{
public:
  MATHLIB_API Vector();
  MATHLIB_API Vector(FLOAT x, FLOAT y, FLOAT z, FLOAT w = 0.0f);
  MATHLIB_API virtual ~Vector();

public:
  MATHLIB_API inline VOID Set(FLOAT x, FLOAT y, FLOAT z, FLOAT w = 0.0f);
  MATHLIB_API inline VOID Set(const VECTOR4D v);
  MATHLIB_API inline VOID Get(VECTOR4D v) const;
  MATHLIB_API inline const FLOAT *Get() const;
  MATHLIB_API virtual inline BOOL GetDynamic();

public:
  MATHLIB_API virtual VOID Update();

public:
  MATHLIB_API FLOAT Magnitude() const;
  MATHLIB_API VOID Null();
  MATHLIB_API BOOL Normalize();
  MATHLIB_API VOID CompletePlaneEquation(const Point &pt);

public:
  MATHLIB_API inline FLOAT &operator[](const UINT_32 uiIndex);
  MATHLIB_API inline FLOAT operator()(const UINT_32 uiIndex) const;

  MATHLIB_API Vector &operator=(const Vector &v);
  MATHLIB_API Vector &operator=(const Point &pt);
  MATHLIB_API Vector &operator+=(const Vector &v);
  MATHLIB_API Vector &operator-=(const Vector &v);
  MATHLIB_API Vector &operator*=(const FLOAT f);
  MATHLIB_API Vector &operator/=(const FLOAT f);

  MATHLIB_API BOOL operator==(const Vector &v) const;
  MATHLIB_API BOOL operator!=(const Vector &v) const;

  MATHLIB_API inline Vector Cross(const Vector &v) const;
  MATHLIB_API FLOAT Dot(const Vector &v) const;

public:
  MATHLIB_API VOID Display() const;

//////////////////////////////////////////////////
// Object Variables
//
protected:
  VECTOR4D m_v;
};

//////////////////////////////////////////////////////////////////////
// External Operators
//

MATHLIB_API inline Vector operator-(const Vector &v);
MATHLIB_API inline Vector operator+(const Vector &v, const Vector &w);
MATHLIB_API inline Vector operator-(const Vector &v, const Vector &w);
MATHLIB_API inline Vector operator*(const Vector &v, const Vector &w);
MATHLIB_API inline Vector operator*(const Vector &v, const FLOAT f);
MATHLIB_API inline Vector operator*(const FLOAT f, const Vector &v);
MATHLIB_API inline Vector operator/(const Vector &v, const FLOAT f);
MATHLIB_API inline Vector operator/(const FLOAT f, const Vector &v);

MATHLIB_API inline Vector ComponentMultiply(const Vector &v, const Vector &w);
MATHLIB_API DOUBLE AngleBetween(const Vector &v1, const Vector &v2);

//////////////////////////////////////////////////////////////////////////////
// Inlined Functions
//

MATHLIB_API inline VOID Vector::Set(FLOAT x, FLOAT y, FLOAT z, FLOAT w/* = 0.0f*/)
{
  m_v[0] = x;
  m_v[1] = y;
  m_v[2] = z;
  m_v[3] = w;
}

MATHLIB_API inline VOID Vector::Set(const VECTOR4D v)
{
  ASSERT( v != NULL );

  memcpy(m_v, v, sizeof(VECTOR4D));
}

MATHLIB_API inline VOID Vector::Get(VECTOR4D v) const
{
  ASSERT( v != NULL );

  memcpy(v, m_v, sizeof(VECTOR4D));
}

MATHLIB_API inline const FLOAT *Vector::Get() const
{
  return m_v;
}

MATHLIB_API inline BOOL Vector::GetDynamic()
{
  return FALSE;
}

MATHLIB_API inline FLOAT &Vector::operator[](const UINT_32 uiIndex)
{
  return m_v[uiIndex];
}

MATHLIB_API inline FLOAT Vector::operator()(const UINT_32 uiIndex) const
{
  return m_v[uiIndex];
}

MATHLIB_API inline Vector Vector::Cross(const Vector &v) const
{
  return (*this) * v;
}

//////////////////////////////////////////////////////////////////////
// External Operator Implementations
//

MATHLIB_API inline Vector operator-(const Vector &v)
{
  return Vector(-v(0), -v(1), -v(2), -v(3));
}

MATHLIB_API inline Vector operator+(const Vector &v, const Vector &w)
{
  return Vector(v(0) + w(0), v(1) + w(1), v(2) + w(2), v(3) + w(3));
}

MATHLIB_API inline Vector operator-(const Vector &v, const Vector &w)
{
  return Vector(v(0) - w(0), v(1) - w(1), v(2) - w(2), v(3) - w(3));
}

MATHLIB_API inline Vector operator*(const Vector &v, const Vector &w)
{
  return Vector(v(1)*w(2) - v(2)*w(1),
		v(2)*w(0) - v(0)*w(2), 
		v(0)*w(1) - v(1)*w(0),  
		0.0);
}

MATHLIB_API inline Vector operator*(const Vector &v, const FLOAT f)
{
  return Vector(v(0)*f, v(1)*f, v(2)*f, v(3)*f);  
}

MATHLIB_API inline Vector operator*(const FLOAT f, const Vector &v)
{
  return Vector(v(0)*f, v(1)*f, v(2)*f, v(3)*f);  
}

MATHLIB_API inline Vector operator/(const Vector &v, const FLOAT f)
{
  ASSERT( f != 0.0 );

  return Vector(v(0)/f, v(1)/f, v(2)/f, v(3)/f);  
}

MATHLIB_API inline Vector operator/(const FLOAT f, const Vector &v)
{
  return Vector((v(0) == 0.0) ? ((FLOAT)0.0) : f/v(0),
		(v(1) == 0.0) ? ((FLOAT)0.0) : f/v(1),
		(v(2) == 0.0) ? ((FLOAT)0.0) : f/v(2),
		(v(3) == 0.0) ? ((FLOAT)0.0) : f/v(3));
}

MATHLIB_API inline Vector ComponentMultiply(const Vector &v, const Vector &w)
{
  return Vector(v(0)*w(0), v(1)*w(1), v(2)*w(2), v(3)*w(3));  
}

#endif // __VECTOR_H__

